//	GeometryGamesPanGestureRecognizer.swift
//
//	© 2022 by Jeff Weeks
//	See TermsOfUse.txt

import UIKit.UIGestureRecognizerSubclass

//	The UIKit's standard UIPanGestureRecognizer,
//	when it reports that a pan gesture has begun,
//	gives the location at which the pan gesture
//	was recognized, not the location at which it began.
//	This subclass records the location at which it began,
//	which for most purposes is more useful (for example,
//	to determine what object the user wants to move).
//
//	Note:  UIPanGestureRecognizer and GeometryGamesPanGestureRecognizer
//	exist only in the context of the UIKit.  Someday,
//	when SwiftUI's gesture handle capabilities have improved,
//	I plan to replace the Geometry Games' UIKit-based gesture handling
//	with SwiftUI-based gesture handling, at which point
//	this file may be archived and removed.
//
class GeometryGamesPanGestureRecognizer: UIPanGestureRecognizer {
	
	var itsInitialTouchLocation = CGPoint(x: 0.0, y: 0.0)
	var itsInitialTouchTime: CFAbsoluteTime = 0.0
	
	override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent) {

		super.touchesBegan(touches, with: event)
		
		if let theFirstTouch = touches.first {
			itsInitialTouchLocation = theFirstTouch.location(in: view)
			itsInitialTouchTime = CFAbsoluteTimeGetCurrent()
		}
	}
}
